# flake8: noqa

# This minimal dataset was available as part of
# pysarpro 0.15 and will be retained until
# further notice.
# Testing data and additional datasets should only
# be made available by pooch
legacy_datasets = [
    'astronaut.png',
    '01-sar/signal1_ac.npy',
    '01-sar/signal1_rc.npy',
    '01-sar/signal2_ac.npy',
    '01-sar/signal2_rc.npy',
]

# Registry of datafiles that can be downloaded along with their SHA256 hashes
# To generate the SHA256 hash, use the command
# openssl sha256 filename
registry = {
    "data/astronaut.png": "88431cd9653ccd539741b555fb0a46b61558b301d4110412b5bc28b5e3ea6cb5",
    "data/01-sar/raw-img.rat": "88316eb411b3099575cb6a813911af99e61d1116486e11c75a1327a0da2b14a2",
    "data/01-sar/signal1_ac.npy": "814e847e979c6ae0c9cda290c7f6b2d7ed2bf212cf8b454a8671abf1f9d5df52",
    "data/01-sar/signal1_rc.npy": "30f43d643b162eda34061b72bf64bc9a4e1893cfdb309e1e577fa9ba19e629af",
    "data/01-sar/signal2_ac.npy": "ed62d3d0e8a7c52f6b031493f537304c77b7fd833dd8820578cb847bbcc42d60",
    "data/01-sar/signal2_rc.npy": "8c3b01775eebfcb1795aaaab9396928c3251c21d9cfe3b0484ce0e9624640138",
    "data/polsar/slc_16afrisr0107_Lhh_tcal_test.rat": "d1b50ede4c0f981ffa32beaf3d355ac2655b0a076e7e48c029c6facb3cea9845",
    "data/polsar/slc_16afrisr0107_Lhh_tcal_test.rat.hdr": "b9dbca4c6f86d14743441782dd09e12127da8a82cb0fea496f2fe3f3654e0804",
    "data/polsar/slc_16afrisr0107_Lhv_tcal_test.rat": "f045e8251d9a38e4e35a303c9929545fc115b71da57176277abffd3d99ba5683",
    "data/polsar/slc_16afrisr0107_Lhv_tcal_test.rat.hdr": "be45012e118e8b5c5a97ce1d43e10208fd5670dd323be023ea59e5e4bf2faaae",
    "data/polsar/slc_16afrisr0107_Lvh_tcal_test.rat": "8304975ca3e523afa4d3f14e5502fa9a647f180b3bbc853dda2740851cfa3f04",
    "data/polsar/slc_16afrisr0107_Lvh_tcal_test.rat.hdr": "ba19984e02c3c13eefa575cc21f96f833082901c38969df051f2ff47c70f34a7",
    "data/polsar/slc_16afrisr0107_Lvv_tcal_test.rat": "5305c2047ef648b9ff3f28547c63472a020429739ab6d8f8be2480cae08e54e2",
    "data/polsar/slc_16afrisr0107_Lvv_tcal_test.rat.hdr": "860bf405b0ce68530f302deea7824d7812cb2e554f904d4de9292b1919d88a25",
    "data/insar/common_profile.npy": "3d7b425801ca4bb55dbd7e994b644317c14570a4ebe20d6c9eeb34dea283119a",
    "data/insar/flat_earth_Mondah_S_2015_11_11_cut.npy": "dc92d9fc3f2d5d68819b4621b0df9c5f92f1e734a8b0e1f765494e4cf71ea12e",
    "data/insar/flat_earth_Mondah_S_2015_11_11_cut.rat": "8e2186e30bca1b275b95d7b9401590ede520085feac8aa6c5f73dbdb1143cdf5",
    "data/insar/image1.rat": "cb8bb55973f85604df57f077c0525e2c529b9832924bcb7115132deedede0d15",
    "data/insar/image2.rat": "7b071c7d69cf6a294b9b8b7187fe4aaef71798f55b4c47c4540e278a71d9449d",
    "data/insar/master_HH_Mondah_S_2015_11_11_cut.npy": "c7521b9fbbe560a08d539bf88d2c42bca58a84061e1ece712b918399c884d00c",
    "data/insar/master_HH_Mondah_S_2015_11_11_cut.rat": "983266a183e44e9b2cee0fa0b445b1d7258b595e0318d29c0a42df19b546b940",
    "data/insar/RH100_Mondah_S_2015_11_11_cut.npy": "f432a757aa1f5759a5b04450a46f6ac2544106ca1c5559483d6386e147b8abcc",
    "data/insar/RH100_Mondah_S_2015_11_11_cut.rat": "ad3d5a08a4d4e38e8904d396c59581a611e2d29d0ac590171dc61ea2018ade71",
    "data/insar/slave_HH_Mondah_S_2015_11_11_cut.npy": "9eaf6d91fd73be12ab310910c7ccbf5fb66651e52d8b976ca69df6aa4198315f",
    "data/insar/slave_HH_Mondah_S_2015_11_11_cut.rat": "b06a4f07197359b47c68a101e17c77e3371e24eaa91854f26da540188148015e",
    "data/polinsar/gamma_max.rat": "ef51aa6486459d64b86fca63cd0a73623a1600945753fa0bc120d10f4de28c46",
    "data/polinsar/gamma_min.rat": "5f86fc60ed7153502c2c971fe359725da091807204aa9d46d57c6443f0384c51",
    "data/polinsar/incidence_15tmpsar0302_L_t01.rat": "8072888fa7ada41ff18317e6de30e2a56d78e0c5665cb087be66f50768d57de6",
    "data/polinsar/kz_2d_demc_15tmpsar0302_15tmpsar0303_t01.rat": "8ad3a16da3ef2c621a47acafceed5d7ac07f5643b53c47d8f94cd1a608d3adb3",
    "data/polinsar/Lida_r1503.rat": "c57bda37ebf97210e16163ac487bb00d5b50fb7d61f2192280c3bc442d524b97",
    "data/polinsar/local_slope_15tmpsar0302_L_t01.rat": "f1326a8f191cd78af3e89f9e29d353449d8c9c16f44f405b09066e69b58f07d8",
    "data/polinsar/pha_flat_15tmpsar0302_15tmpsar0303_Lhh_t01.rat": "913a75e680d56d04f4fcd2f5e0dea0786b7236d3bd749c50330719f333a4fea4",
    "data/polinsar/pha_max.rat": "4bd6d30f78b5164ab8a39a31eb88e3fad847695f20851c32cb67cb498b44229e",
    "data/polinsar/pha_min.rat": "bcb3631abaa1898dd0322822544da9b245638769c8caa0f86940145a657415b1",
    "data/polinsar/slc_15tmpsar0302_Lhh_t01.rat": "aef646a151f897d49ff9f68bfe0caf504db431eb1ba1430a4a9874647f52506a",
    "data/polinsar/slc_15tmpsar0302_Lhv_t01.rat": "fdd503474723f678e967537b4155e99a4a7a8d2019b8a1591fec5d87a794cadc",
    "data/polinsar/slc_15tmpsar0302_Lvh_t01.rat": "8a5d247404d108f03fdd94bc417bc8f9d0f209f62f09bcd5e3b41f9a31ade412",
    "data/polinsar/slc_15tmpsar0302_Lvv_t01.rat": "1a6a38cd0ff2d313c02c812f48ce800f14b5caf1ceb462554dbd9ce93354933d",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lhh_t01.rat": "53dc085de9dd6b828248e54aff96d6c3dc47dd87c5ea196ba36f62f711b8ee58",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lhv_t01.rat": "4bd6b729c01b521fbea3d593828d894aff6d8e130d82fad3fadc2497d441ce01",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lvh_t01.rat": "7851e089feb307654bb912609ee8d2fe88e274bec0fe31f64693c59d732e8923",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lvv_t01.rat": "03121641edd8f48d355a310c41178f01bb92e35e76c684853c993d6db80d27bd",
    "data/tomosar/dem.rat": "226f6806773b1759ddb696b25dabb44ff776bcf727e3a33edc9f3cf41a02b229",
    "data/tomosar/kz_00.rat": "f915c9980d71c389aa9d850041d08cea8eff2fdc3443cd72301912acf5513b4b",
    "data/tomosar/kz_01.rat": "95b6be70d5e79dcd87bce1762c3f86e0131327625ee47242d186a9c440cc4039",
    "data/tomosar/kz_02.rat": "daab75242a800e6856d51bda9fe787a2247fa8a065c05b0da1600619a2dc5d57",
    "data/tomosar/kz_03.rat": "1a0ebde5c5b5d2709f05519de0a8b55bbf8589f4ae3da07b7e0f38a962531e3d",
    "data/tomosar/kz_04.rat": "7bbcd6d997cfe93befb05b9b33352030ba6d4a37bd7a508e38ca7028043c1bd1",
    "data/tomosar/kz_05.rat": "ad2dbd76e1ce7b55fdfe441714f86a4576d4166ee607f0e6594dbb1ac3a0dbd2",
    "data/tomosar/kz_06.rat": "0031fd2d43027a770c1052fd843e734beebe58899ac99656152e7f1fe229a653",
    "data/tomosar/kz_07.rat": "a26996d35359b1ed3a72d89abde140d1999939f8694705962e7e8fbe57a88381",
    "data/tomosar/kz_08.rat": "9c5e8cb255412d79059f464ac3b658a1eb6ff9c8136b8fca4761a98712071d45",
    "data/tomosar/kz_09.rat": "768af65c80338384ce881770004e016e6be4f544912ff98cbd9e209a01eabcd4",
    "data/tomosar/kz_10.rat": "989d57637ff9eb448de82de7419f1506bcaa3a4958ca2c77a4c2d11c127ae61d",
    "data/tomosar/lidar_chm.rat": "96f4e56ca8e3d20f2868bae6d77a4075139d2f26d1609f1281e8f68db8b0b15b",
    "data/tomosar/lidar_dtm.rat": "c769673c01ff6be88f8ffcb6e5cb870ad5b9ec7e70a5f3b265cbff9e5b2e577f",
    "data/tomosar/pha_dem_00.rat": "380e0633010eeda664d1cf52e09d57fb1660fe84edd79110190a09368fd9cca6",
    "data/tomosar/pha_dem_01.rat": "57db905f21e7e6d4c1b6cb13ffb6be5aacddf54b6912cc084f90bb35fac4b371",
    "data/tomosar/pha_dem_02.rat": "b6fc41ee2ed666b54d90b48890c6a36ca8347e1ab3e053473ce6e57520442753",
    "data/tomosar/pha_dem_03.rat": "b69aa4889c3c2a4cbef817c0acf556242bfbbeb0820eac038608c3c99f913a36",
    "data/tomosar/pha_dem_04.rat": "a6bdffa10c88b3fd1fa2f4562e3e5ac9de11abcd8ec6988636a0e429e57a2f6b",
    "data/tomosar/pha_dem_05.rat": "d253cdbb3eb8553ddba81339c8f74f994d7d0955df6b290e85f93118456043aa",
    "data/tomosar/pha_dem_06.rat": "ba0d99a6d608c84987491fdc20ecd790a43a336b1625062baa178bc6325d8c3a",
    "data/tomosar/pha_dem_07.rat": "a6e061b0e8e7d12629cf9327bb9c161e0cb85eaf75369176f6298aeb445e9605",
    "data/tomosar/pha_dem_08.rat": "43dc7d19985e4e5c8b3d5dd32c4e3e31cb363ca88be831d3dd7a7f2d68c62883",
    "data/tomosar/pha_dem_09.rat": "2581da4e098f57de2620015d2b9cdf74bba192032ce07a43b5dee1923f695f27",
    "data/tomosar/pha_dem_10.rat": "672245b9cb33014ae139b7be13775e689cd930c240013e55cd55498e0657ddce",
    "data/tomosar/pos_scatters_ground.npy": "a1902c3b0c428587ce1043a370852a2eb80d82eef4b969262b9675562d98c05e",
    "data/tomosar/pos_scatters_layer1.npy": "9ec4bb41928f880eb8eac2697e37732e5c853220e1f3def54ac07caf8b277dfd",
    "data/tomosar/slc_00_HH.rat": "b434350d2cf9c8c1232af6ef18b474ce4683ca1d84608b33f759ed6f3e872331",
    "data/tomosar/slc_00_HV.rat": "a32cc617ca51a8dda50b19a8d67f08d99cd6c77a3fab356cfef9c7c3d3ee81a7",
    "data/tomosar/slc_00_VV.rat": "b37ff61e689b3abe0bbf83c323b524e4ad7e40b3ae8d69d6e97602f2f4aebfad",
    "data/tomosar/slc_01_HH.rat": "bb06db40651aa757e281332c2f290d87593e79a98cd1c6c3a62f22f38a375015",
    "data/tomosar/slc_01_HV.rat": "0978b94b23ba36af92e4e1834263c1001daa5f6111d3fe6e761bf1aa7fb1a464",
    "data/tomosar/slc_01_VV.rat": "6eec3ecb5204dbc8a9bb0781daba7331d50dae18ea953f373e437be7054f9a32",
    "data/tomosar/slc_02_HH.rat": "cb57062d39f0f008d1a172b8c3f8ce311c9140d8e464e395d75e219755bd095a",
    "data/tomosar/slc_02_HV.rat": "f6e8c065f09030e4c733ae8128c4337a737d64a9da76ce422c5e45cf0e74f53f",
    "data/tomosar/slc_02_VV.rat": "33bbe3e0480891e309bb42c2b0407690fbf536178477d12ff18101168bf1b772",
    "data/tomosar/slc_03_HH.rat": "19c488eeffae4496f5bbea2e26df214a59b5f0ae7bcbed674d9db9dfdcc03184",
    "data/tomosar/slc_03_HV.rat": "bd11da7f59fdc5afb59b97901cd08db411ec927ebe7d42cc7da28b8ea6faa608",
    "data/tomosar/slc_03_VV.rat": "6f648dac736b541cc53023a9ec635275eb5ff40eecc28ea0f69fd373a7c8ed13",
    "data/tomosar/slc_04_HH.rat": "5ed7c8d8c9710779f32c8530ed1ff2151cb209693a75afeff3bb416da12f0a55",
    "data/tomosar/slc_04_HV.rat": "7d1a1267a010e00a60a7aae595155ccb6c3e80069689fd1cb0f999d60fc01778",
    "data/tomosar/slc_04_VV.rat": "539bfe527f5452d7e44f11192dd13e9d9d2387536ed05aea250f4188bddc8495",
    "data/tomosar/slc_05_HH.rat": "ce28edb56fe6affbe042168b83bee5c7bb4d6d2e671490cd91a7ddd3fa8fc7a5",
    "data/tomosar/slc_05_HV.rat": "24ebea0807e228dc4aaff1eed8346e77f79c7249a5680d5c8223aae0eef913a4",
    "data/tomosar/slc_05_VV.rat": "7959950f0b673becce3a47aded871817a087093220554508bbc7995eda3fcf3c",
    "data/tomosar/slc_06_HH.rat": "d679258e5324ee37a7400728b3f75a508590b5bf86b75df4bb9b02731c6c4ee6",
    "data/tomosar/slc_06_HV.rat": "2f3f173af1f1b5d3b45bfeb6c411b80a96b6d0e90a205358f7c52a43a524d3a6",
    "data/tomosar/slc_06_VV.rat": "cd20797422ce0eb0e09270e77b9a2d0ac96c80ef9f08c237373de865e01a6571",
    "data/tomosar/slc_07_HH.rat": "fcd7bb701e68749f6844f73fa34a6321cbb0b362518d7bcb376a4d31c1ef3d7e",
    "data/tomosar/slc_07_HV.rat": "332a96a55951b703c649719b178e5c46690712f31d8052c85a1d7e0fe67077fe",
    "data/tomosar/slc_07_VV.rat": "703897c7ed72dc698b8ff58ce26674318a453847a2b1d5218e26d527c5827a4e",
    "data/tomosar/slc_08_HH.rat": "7a6e9cc764888b0df3d2a61d381c7096f5f87b11070dd450270849108a9bfd2e",
    "data/tomosar/slc_08_HV.rat": "50f08a8e20ff55862f10938bbc54dc947679d08933685b2f6c63b3e42dcdeeeb",
    "data/tomosar/slc_08_VV.rat": "d172c4a5d03b71fb2626b8b49245f93f2bf22c22509513014a210bdb4db61ea5",
    "data/tomosar/slc_09_HH.rat": "e122bb4b9418d93153cb8aec40c70061abafa0d7ed9cab839931b47f50d3d3a0",
    "data/tomosar/slc_09_HV.rat": "40057ad2b49492c318d7eb3f5edbd47594a611dcfba5d3f4a38b6f3535233a70",
    "data/tomosar/slc_09_VV.rat": "e381df66217995d7a79702eea4082ae3ba960ba761780ba35e755a4df06c03da",
    "data/tomosar/slc_10_HH.rat": "bd34eef79940c00ccafc80f637c6d979f90b79d1ee301d6327b59233f320dc9a",
    "data/tomosar/slc_10_HV.rat": "b435efcf0366f3bcf62ac57c81b371e60879edfac001ef2fad0452ce3200f24b",
    "data/tomosar/slc_10_VV.rat": "dac25b527e7e223861311a5fa81f18919ac9d9f9c77d1c1753f41d1070dd9c3f",
    "data/DInSAR.zip": "14692fb707283319e645bda92a6924351528c149d731d2fbc63c72cf3e445f1c",

}

registry_urls = {
    "data/01-sar/raw-img.rat": "https://zenodo.org/records/10797182/files/raw-img.rat",
    "data/polsar/slc_16afrisr0107_Lhh_tcal_test.rat": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lhh_tcal_test.rat",
    "data/polsar/slc_16afrisr0107_Lhh_tcal_test.rat.hdr": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lhh_tcal_test.rat.hdr",
    "data/polsar/slc_16afrisr0107_Lhv_tcal_test.rat": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lhv_tcal_test.rat",
    "data/polsar/slc_16afrisr0107_Lhv_tcal_test.rat.hdr": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lhv_tcal_test.rat.hdr",
    "data/polsar/slc_16afrisr0107_Lvh_tcal_test.rat": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lvh_tcal_test.rat",
    "data/polsar/slc_16afrisr0107_Lvh_tcal_test.rat.hdr": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lvh_tcal_test.rat.hdr",
    "data/polsar/slc_16afrisr0107_Lvv_tcal_test.rat": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lvv_tcal_test.rat",
    "data/polsar/slc_16afrisr0107_Lvv_tcal_test.rat.hdr": "https://zenodo.org/records/10797182/files/slc_16afrisr0107_Lvv_tcal_test.rat.hdr",
    "data/insar/common_profile.npy": "https://zenodo.org/records/10797182/files/common_profile.npy",
    "data/insar/flat_earth_Mondah_S_2015_11_11_cut.npy": "https://zenodo.org/records/10797182/files/flat_earth_Mondah_S_2015_11_11_cut.npy",
    "data/insar/flat_earth_Mondah_S_2015_11_11_cut.rat": "https://zenodo.org/records/10797182/files/flat_earth_Mondah_S_2015_11_11_cut.rat",
    "data/insar/image1.rat": "https://zenodo.org/records/10797182/files/image1.rat",
    "data/insar/image2.rat": "https://zenodo.org/records/10797182/files/image2.rat",
    "data/insar/master_HH_Mondah_S_2015_11_11_cut.npy": "https://zenodo.org/records/10797182/files/master_HH_Mondah_S_2015_11_11_cut.npy",
    "data/insar/master_HH_Mondah_S_2015_11_11_cut.rat": "https://zenodo.org/records/10797182/files/master_HH_Mondah_S_2015_11_11_cut.rat",
    "data/insar/RH100_Mondah_S_2015_11_11_cut.npy": "https://zenodo.org/records/10797182/files/RH100_Mondah_S_2015_11_11_cut.npy",
    "data/insar/RH100_Mondah_S_2015_11_11_cut.rat": "https://zenodo.org/records/10797182/files/RH100_Mondah_S_2015_11_11_cut.rat",
    "data/insar/slave_HH_Mondah_S_2015_11_11_cut.npy": "https://zenodo.org/records/10797182/files/slave_HH_Mondah_S_2015_11_11_cut.npy",
    "data/insar/slave_HH_Mondah_S_2015_11_11_cut.rat": "https://zenodo.org/records/10797182/files/slave_HH_Mondah_S_2015_11_11_cut.rat",
    "data/polinsar/gamma_max.rat": "https://zenodo.org/records/10797182/files/gamma_max.rat",
    "data/polinsar/gamma_min.rat": "https://zenodo.org/records/10797182/files/gamma_min.rat",
    "data/polinsar/incidence_15tmpsar0302_L_t01.rat": "https://zenodo.org/records/10797182/files/incidence_15tmpsar0302_L_t01.rat",
    "data/polinsar/kz_2d_demc_15tmpsar0302_15tmpsar0303_t01.rat": "https://zenodo.org/records/10797182/files/kz_2d_demc_15tmpsar0302_15tmpsar0303_t01.rat",
    "data/polinsar/Lida_r1503.rat": "https://zenodo.org/records/10797182/files/Lida_r1503.rat",
    "data/polinsar/local_slope_15tmpsar0302_L_t01.rat": "https://zenodo.org/records/10797182/files/local_slope_15tmpsar0302_L_t01.rat",
    "data/polinsar/pha_flat_15tmpsar0302_15tmpsar0303_Lhh_t01.rat": "https://zenodo.org/records/10797182/files/pha_flat_15tmpsar0302_15tmpsar0303_Lhh_t01.rat",
    "data/polinsar/pha_max.rat": "https://zenodo.org/records/10797182/files/pha_max.rat",
    "data/polinsar/pha_min.rat": "https://zenodo.org/records/10797182/files/pha_min.rat",
    "data/polinsar/slc_15tmpsar0302_Lhh_t01.rat": "https://zenodo.org/records/10797182/files/slc_15tmpsar0302_Lhh_t01.rat",
    "data/polinsar/slc_15tmpsar0302_Lhv_t01.rat": "https://zenodo.org/records/10797182/files/slc_15tmpsar0302_Lhv_t01.rat",
    "data/polinsar/slc_15tmpsar0302_Lvh_t01.rat": "https://zenodo.org/records/10797182/files/slc_15tmpsar0302_Lvh_t01.rat",
    "data/polinsar/slc_15tmpsar0302_Lvv_t01.rat": "https://zenodo.org/records/10797182/files/slc_15tmpsar0302_Lvv_t01.rat",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lhh_t01.rat": "https://zenodo.org/records/10797182/files/slc_coreg_15tmpsar0302_15tmpsar0303_Lhh_t01.rat",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lhv_t01.rat": "https://zenodo.org/records/10797182/files/slc_coreg_15tmpsar0302_15tmpsar0303_Lhv_t01.rat",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lvh_t01.rat": "https://zenodo.org/records/10797182/files/slc_coreg_15tmpsar0302_15tmpsar0303_Lvh_t01.rat",
    "data/polinsar/slc_coreg_15tmpsar0302_15tmpsar0303_Lvv_t01.rat": "https://zenodo.org/records/10797182/files/slc_coreg_15tmpsar0302_15tmpsar0303_Lvv_t01.rat",
    "data/tomosar/dem.rat": "https://zenodo.org/records/10797182/files/dem.rat",
    "data/tomosar/kz_00.rat": "https://zenodo.org/records/10797182/files/kz_00.rat",
    "data/tomosar/kz_01.rat": "https://zenodo.org/records/10797182/files/kz_01.rat",
    "data/tomosar/kz_02.rat": "https://zenodo.org/records/10797182/files/kz_02.rat",
    "data/tomosar/kz_03.rat": "https://zenodo.org/records/10797182/files/kz_03.rat",
    "data/tomosar/kz_04.rat": "https://zenodo.org/records/10797182/files/kz_04.rat",
    "data/tomosar/kz_05.rat": "https://zenodo.org/records/10797182/files/kz_05.rat",
    "data/tomosar/kz_06.rat": "https://zenodo.org/records/10797182/files/kz_06.rat",
    "data/tomosar/kz_07.rat": "https://zenodo.org/records/10797182/files/kz_07.rat",
    "data/tomosar/kz_08.rat": "https://zenodo.org/records/10797182/files/kz_08.rat",
    "data/tomosar/kz_09.rat": "https://zenodo.org/records/10797182/files/kz_09.rat",
    "data/tomosar/kz_10.rat": "https://zenodo.org/records/10797182/files/kz_10.rat",
    "data/tomosar/lidar_chm.rat": "https://zenodo.org/records/10797182/files/lidar_chm.rat",
    "data/tomosar/lidar_dtm.rat": "https://zenodo.org/records/10797182/files/lidar_dtm.rat",
    "data/tomosar/pha_dem_00.rat": "https://zenodo.org/records/10797182/files/pha_dem_00.rat",
    "data/tomosar/pha_dem_01.rat": "https://zenodo.org/records/10797182/files/pha_dem_01.rat",
    "data/tomosar/pha_dem_02.rat": "https://zenodo.org/records/10797182/files/pha_dem_02.rat",
    "data/tomosar/pha_dem_03.rat": "https://zenodo.org/records/10797182/files/pha_dem_03.rat",
    "data/tomosar/pha_dem_04.rat": "https://zenodo.org/records/10797182/files/pha_dem_04.rat",
    "data/tomosar/pha_dem_05.rat": "https://zenodo.org/records/10797182/files/pha_dem_05.rat",
    "data/tomosar/pha_dem_06.rat": "https://zenodo.org/records/10797182/files/pha_dem_06.rat",
    "data/tomosar/pha_dem_07.rat": "https://zenodo.org/records/10797182/files/pha_dem_07.rat",
    "data/tomosar/pha_dem_08.rat": "https://zenodo.org/records/10797182/files/pha_dem_08.rat",
    "data/tomosar/pha_dem_09.rat": "https://zenodo.org/records/10797182/files/pha_dem_09.rat",
    "data/tomosar/pha_dem_10.rat": "https://zenodo.org/records/10797182/files/pha_dem_10.rat",
    "data/tomosar/pos_scatters_ground.npy": "https://zenodo.org/records/10797182/files/pos_scatters_ground.npy",
    "data/tomosar/pos_scatters_layer1.npy": "https://zenodo.org/records/10797182/files/pos_scatters_layer1.npy",
    "data/tomosar/slc_00_HH.rat": "https://zenodo.org/records/10797182/files/slc_00_HH.rat",
    "data/tomosar/slc_00_HV.rat": "https://zenodo.org/records/10797182/files/slc_00_HV.rat",
    "data/tomosar/slc_00_VV.rat": "https://zenodo.org/records/10797182/files/slc_00_VV.rat",
    "data/tomosar/slc_01_HH.rat": "https://zenodo.org/records/10797182/files/slc_01_HH.rat",
    "data/tomosar/slc_01_HV.rat": "https://zenodo.org/records/10797182/files/slc_01_HV.rat",
    "data/tomosar/slc_01_VV.rat": "https://zenodo.org/records/10797182/files/slc_01_VV.rat",
    "data/tomosar/slc_02_HH.rat": "https://zenodo.org/records/10797182/files/slc_02_HH.rat",
    "data/tomosar/slc_02_HV.rat": "https://zenodo.org/records/10797182/files/slc_02_HV.rat",
    "data/tomosar/slc_02_VV.rat": "https://zenodo.org/records/10797182/files/slc_02_VV.rat",
    "data/tomosar/slc_03_HH.rat": "https://zenodo.org/records/10797182/files/slc_03_HH.rat",
    "data/tomosar/slc_03_HV.rat": "https://zenodo.org/records/10797182/files/slc_03_HV.rat",
    "data/tomosar/slc_03_VV.rat": "https://zenodo.org/records/10797182/files/slc_03_VV.rat",
    "data/tomosar/slc_04_HH.rat": "https://zenodo.org/records/10797182/files/slc_04_HH.rat",
    "data/tomosar/slc_04_HV.rat": "https://zenodo.org/records/10797182/files/slc_04_HV.rat",
    "data/tomosar/slc_04_VV.rat": "https://zenodo.org/records/10797182/files/slc_04_VV.rat",
    "data/tomosar/slc_05_HH.rat": "https://zenodo.org/records/10797182/files/slc_05_HH.rat",
    "data/tomosar/slc_05_HV.rat": "https://zenodo.org/records/10797182/files/slc_05_HV.rat",
    "data/tomosar/slc_05_VV.rat": "https://zenodo.org/records/10797182/files/slc_05_VV.rat",
    "data/tomosar/slc_06_HH.rat": "https://zenodo.org/records/10797182/files/slc_06_HH.rat",
    "data/tomosar/slc_06_HV.rat": "https://zenodo.org/records/10797182/files/slc_06_HV.rat",
    "data/tomosar/slc_06_VV.rat": "https://zenodo.org/records/10797182/files/slc_06_VV.rat",
    "data/tomosar/slc_07_HH.rat": "https://zenodo.org/records/10797182/files/slc_07_HH.rat",
    "data/tomosar/slc_07_HV.rat": "https://zenodo.org/records/10797182/files/slc_07_HV.rat",
    "data/tomosar/slc_07_VV.rat": "https://zenodo.org/records/10797182/files/slc_07_VV.rat",
    "data/tomosar/slc_08_HH.rat": "https://zenodo.org/records/10797182/files/slc_08_HH.rat",
    "data/tomosar/slc_08_HV.rat": "https://zenodo.org/records/10797182/files/slc_08_HV.rat",
    "data/tomosar/slc_08_VV.rat": "https://zenodo.org/records/10797182/files/slc_08_VV.rat",
    "data/tomosar/slc_09_HH.rat": "https://zenodo.org/records/10797182/files/slc_09_HH.rat",
    "data/tomosar/slc_09_HV.rat": "https://zenodo.org/records/10797182/files/slc_09_HV.rat",
    "data/tomosar/slc_09_VV.rat": "https://zenodo.org/records/10797182/files/slc_09_VV.rat",
    "data/tomosar/slc_10_HH.rat": "https://zenodo.org/records/10797182/files/slc_10_HH.rat",
    "data/tomosar/slc_10_HV.rat": "https://zenodo.org/records/10797182/files/slc_10_HV.rat",
    "data/tomosar/slc_10_VV.rat": "https://zenodo.org/records/10797182/files/slc_10_VV.rat",
    "data/DInSAR.zip": "https://zenodo.org/records/11171238/files/DInSAR.zip",

}


legacy_registry = {
    ('data/' + filename): registry['data/' + filename] for filename in legacy_datasets
}
